package java3.util3;

import java3.theories.TArrays;

/**
 * http://grepcode.com/file/repository.grepcode.com/java/root/jdk/openjdk/6-b14/java/util/Arrays.java
 * 
 * This class contains various methods for manipulating arrays (such as
 * sorting and searching).  This class also contains a static factory
 * that allows arrays to be viewed as lists.
 *
 * <p>The methods in this class all throw a <tt>NullPointerException</tt> if
 * the specified array reference is null, except where noted.
 *
 * <p>The documentation for the methods contained in this class includes
 * briefs description of the <i>implementations</i>.  Such descriptions should
 * be regarded as <i>implementation notes</i>, rather than parts of the
 * <i>specification</i>.  Implementors should feel free to substitute other
 * algorithms, so long as the specification itself is adhered to.  (For
 * example, the algorithm used by <tt>sort(Object[])</tt> does not have to be
 * a mergesort, but it does have to be <i>stable</i>.)
 *
 * <p>This class is a member of the
 * <a href="{@docRoot}/../technotes/guides/collections/index.html">
 * Java Collections Framework</a>.
 *
 * @author  Josh Bloch
 * @author  Neal Gafter
 * @author  John Rose
 * @since   1.2
 */
public abstract class Arrays {

	/**
	 * Searches the specified array of ints for the specified value using the
	 * binary search algorithm. The array must be sorted (as by the
	 * {@link #sort(int[])} method) prior to making this call. If it is not
	 * sorted, the results are undefined. If the array contains multiple
	 * elements with the specified value, there is no guarantee which one will
	 * be found.
	 * 
	 * @param a
	 *            the array to be searched
	 * @param key
	 *            the value to be searched for
	 * @return index of the search key, if it is contained in the array;
	 *         otherwise, <tt>(-(<i>insertion point</i>) - 1)</tt>. The
	 *         <i>insertion point</i> is defined as the point at which the key
	 *         would be inserted into the array: the index of the first element
	 *         greater than the key, or <tt>a.length</tt> if all elements in the
	 *         array are less than the specified key. Note that this guarantees
	 *         that the return value will be &gt;= 0 if and only if the key is
	 *         found.
	 */
	/*@ 
	@ requires a != null; 
	@*/
	public static int binarySearch(int[] a, int key) {
		return binarySearch0(a, 0, a.length, key);
	}

	/**
	 * Searches a range of the specified array of ints for the specified value
	 * using the binary search algorithm. The range must be sorted (as by the
	 * {@link #sort(int[], int, int)} method) prior to making this call. If it
	 * is not sorted, the results are undefined. If the range contains multiple
	 * elements with the specified value, there is no guarantee which one will
	 * be found.
	 * 
	 * @param a
	 *            the array to be searched
	 * @param fromIndex
	 *            the index of the first element (inclusive) to be searched
	 * @param toIndex
	 *            the index of the last element (exclusive) to be searched
	 * @param key
	 *            the value to be searched for
	 * @return index of the search key, if it is contained in the array within
	 *         the specified range; otherwise,
	 *         <tt>(-(<i>insertion point</i>) - 1)</tt>. The <i>insertion
	 *         point</i> is defined as the point at which the key would be
	 *         inserted into the array: the index of the first element in the
	 *         range greater than the key, or <tt>toIndex</tt> if all elements
	 *         in the range are less than the specified key. Note that this
	 *         guarantees that the return value will be &gt;= 0 if and only if
	 *         the key is found.
	 * @throws IllegalArgumentException
	 *             if {@code fromIndex > toIndex}
	 * @throws ArrayIndexOutOfBoundsException
	 *             if {@code fromIndex < 0 or toIndex > a.length}
	 * @since 1.6
	 */
	public static int binarySearch(int[] a, int fromIndex, int toIndex, int key) {
		rangeCheck(a.length, fromIndex, toIndex);
		return binarySearch0(a, fromIndex, toIndex, key);
	}

	/*@
	@ requires a!=null;
	@ requires TArrays.within(a,fromIndex,toIndex);
	@ requires TArrays.sorted(a,fromIndex,toIndex);
	@
	@ ensures \result >= 0 ==> a[\result] == key;
	@ ensures \result < 0 ==> (!TArrays.in(a,fromIndex,toIndex,key));
	@*/
	private static/*@ pure @*/int binarySearch0(int[] a, int fromIndex,
			int toIndex, int key) {

		if (a == null)
			return 0;

		if (!TArrays.within(a, fromIndex, toIndex))
			return 0;

		if (!TArrays.sorted(a, fromIndex, toIndex))
			return 0;

		int low = fromIndex;
		int high = toIndex - 1;

		binarySearch0_loop_invariant_1(low, high, toIndex, fromIndex, key, a);
		while (low <= high) {
			binarySearch0_loop_invariant_1(low, high, toIndex, fromIndex, key,
					a);

			int mid = low + ((high - low) / 2); // equivalent to (low + high) >>> 1;
			int midVal = a[mid];

			if (midVal < key) {
				low = mid + 1;
			} else if (midVal > key) {
				high = mid - 1;
			} else {
				binarySearch0_loop_invariant_1(low, high, toIndex, fromIndex,
						key, a);
				return mid; // key found
			}

			binarySearch0_loop_invariant_1(low, high, toIndex, fromIndex, key,
					a);
		}
		binarySearch0_loop_invariant_1(low, high, toIndex, fromIndex, key, a);
		return -(low + 1); // key not found.

	}

	private static void binarySearch0_loop_invariant_1(int low, int high,
			int toIndex, int fromIndex, int key, int[] a) {
		/*STUB*/
	}

	/**
	 * Copies the specified array, truncating or padding with zeros (if
	 * necessary) so the copy has the specified length. For all indices that are
	 * valid in both the original array and the copy, the two arrays will
	 * contain identical values. For any indices that are valid in the copy but
	 * not the original, the copy will contain <tt>0</tt>. Such indices will
	 * exist if and only if the specified length is greater than that of the
	 * original array.
	 * 
	 * @param original
	 *            the array to be copied
	 * @param newLength
	 *            the length of the copy to be returned
	 * @return a copy of the original array, truncated or padded with zeros to
	 *         obtain the specified length
	 * @throws NegativeArraySizeException
	 *             if <tt>newLength</tt> is negative
	 * @throws NullPointerException
	 *             if <tt>original</tt> is null
	 * @since 1.6
	 */
	/*@ 
	@ requires original != null;
	@ requires newLength >= 0;
	@ 
	@ assignable original[*];
	@
	@ ensures \result != null;
	@ ensures (\forall int i; 0 <= i && i < original.length ==> original[i] == \old(original[i]));
	@ ensures (\forall int i; original.length <= i && i < newLength ==> \result[i] == 0);
	@ ensures \result.length == newLength;
	@ ensures \result.length <= original.length ==>
	@                  (\forall int i; 0 <= i && i < \result.length ==> \result[i] == original[i]);
	@ ensures \result.length > original.length ==>
	@                  (\forall int i; 0 <= i && i < original.length ==> \result[i] == original[i]);
	@*/
	public static int[] copyOf(int[] original, int newLength) {
		int[] copy = new int[newLength];
		int min_length = Math.min(original.length, newLength);
		java3.lang3.System.arraycopy(original, 0, copy, 0, min_length);
		return copy;
	}

	/*@ 
	@ requires original != null;
	@ requires newLength >= 0;
	@ 
	@ assignable original[*];
	@
	@ ensures \result != null;
	@ ensures (\forall int i; 0 <= i && i < original.length ==> original[i] == \old(original[i]));
	@ ensures (\forall int i; original.length <= i && i < newLength ==> \result[i] == null);
	@ ensures \result.length == newLength;
	@ ensures \result.length <= original.length ==>
	@                  (\forall int i; 0 <= i && i < \result.length ==> \result[i] == original[i]);
	@ ensures \result.length > original.length ==>
	@                  (\forall int i; 0 <= i && i < original.length ==> \result[i] == original[i]);
	@*/
	public static java3.lang3.Integer[] copyOf(java3.lang3.Integer[] original,
			int newLength) {
		java3.lang3.Integer[] copy = new java3.lang3.Integer[newLength];
		java3.lang3.System.arraycopyInteger(original, 0, copy, 0,
				Math.min(original.length, newLength));
		return copy;
	}

	/**
	 * Copies the specified range of the specified array into a new array. The
	 * initial index of the range (<tt>from</tt>) must lie between zero and
	 * <tt>original.length</tt>, inclusive. The value at <tt>original[from]</tt>
	 * is placed into the initial element of the copy (unless
	 * <tt>from == original.length</tt> or <tt>from == to</tt>). Values from
	 * subsequent elements in the original array are placed into subsequent
	 * elements in the copy. The final index of the range (<tt>to</tt>), which
	 * must be greater than or equal to <tt>from</tt>, may be greater than
	 * <tt>original.length</tt>, in which case <tt>0</tt> is placed in all
	 * elements of the copy whose index is greater than or equal to
	 * <tt>original.length - from</tt>. The length of the returned array will be
	 * <tt>to - from</tt>.
	 * 
	 * @param original
	 *            the array from which a range is to be copied
	 * @param from
	 *            the initial index of the range to be copied, inclusive
	 * @param to
	 *            the final index of the range to be copied, exclusive. (This
	 *            index may lie outside the array.)
	 * @return a new array containing the specified range from the original
	 *         array, truncated or padded with zeros to obtain the required
	 *         length
	 * @throws ArrayIndexOutOfBoundsException
	 *             if {@code from < 0} or {@code from > original.length}
	 * @throws IllegalArgumentException
	 *             if <tt>from &gt; to</tt>
	 * @throws NullPointerException
	 *             if <tt>original</tt> is null
	 * @since 1.6
	 */
	public static int[] copyOfRange(int[] original, int from, int to) {
		int newLength = to - from;
		if (newLength < 0)
			throw new java3.lang3.IllegalArgumentException();
		int[] copy = new int[newLength];
		java3.lang3.System.arraycopy(original, from, copy, 0,
				Math.min(original.length - from, newLength));
		return copy;
	}

	/**
	 * Assigns the specified int value to each element of the specified range of
	 * the specified array of ints. The range to be filled extends from index
	 * <tt>fromIndex</tt>, inclusive, to index <tt>toIndex</tt>, exclusive. (If
	 * <tt>fromIndex==toIndex</tt>, the range to be filled is empty.)
	 * 
	 * @param a
	 *            the array to be filled
	 * @param fromIndex
	 *            the index of the first element (inclusive) to be filled with
	 *            the specified value
	 * @param toIndex
	 *            the index of the last element (exclusive) to be filled with
	 *            the specified value
	 * @param val
	 *            the value to be stored in all elements of the array
	 * @throws IllegalArgumentException
	 *             if <tt>fromIndex &gt; toIndex</tt>
	 * @throws ArrayIndexOutOfBoundsException
	 *             if <tt>fromIndex &lt; 0</tt> or
	 *             <tt>toIndex &gt; a.length</tt>
	 */
	/*@ normal_behavior
	  @ requires a != null;
	  @ requires TArrays.within(a, fromIndex, toIndex);
	  @ assignable a[*];
	  @ ensures (\forall int j; 0 <= j && j < fromIndex ==> a[j] == \old(a[j]));
	  @ ensures TArrays.eq(a, fromIndex, toIndex, val);
	  @ ensures (\forall int j; toIndex <= j && j < a.length ==> a[j] == \old(a[j]));
	  @
	  @ also	exceptional_behavior
	  @ requires fromIndex > toIndex;
	  @ signals (java3.lang3.IllegalArgumentException ex) true;
	  @
	  @ also exceptional_behavior
	  @ requires fromIndex < 0 || toIndex > a.length;
	  @ signals (java3.lang3.ArrayIndexOutOfBoundsException  ex) true;
	  @
	  @ also exceptional_behavior
	  @ requires a == null;
	  @ signals (NullPointerException ex) true;
	  @*/
	public static void fill1(int[] a, int fromIndex, int toIndex, int val)
			throws java3.lang3.IllegalArgumentException,
			java3.lang3.ArrayIndexOutOfBoundsException, NullPointerException {
		if (a == null)
			throw new NullPointerException();

		int[] old_a = new int[a.length];
		for (int i = 0; i < a.length; i++) {
			old_a[i] = a[i];
		}

		rangeCheck(a.length, fromIndex, toIndex);

		int ic = fromIndex;

		int loop_counter_1 = 0;
		fill1_loop_invariant_1(ic, fromIndex, toIndex, a, val, old_a);
		for (; ic < toIndex;) {
			fill1_loop_invariant_1(ic, fromIndex, toIndex, a, val, old_a);
			if (loop_counter_1 < 10) {
				loop_counter_1++;
			}
			a[ic] = val;
			ic++;
			fill1_loop_invariant_1(ic, fromIndex, toIndex, a, val, old_a);
		}
		fill1_loop_invariant_1(ic, fromIndex, toIndex, a, val, old_a);
	}

	private static void fill1_loop_invariant_1(int ic, int fromIndex,
			int toIndex, int[] a, int val, int[] old_a) {
		/*STUB*/
	}

	/**
	 * Assigns the specified int value to each element of the specified array of
	 * ints.
	 * 
	 * @param a
	 *            the array to be filled
	 * @param val
	 *            the value to be stored in all elements of the array
	 */
	/*@ 
	@ requires a != null;
	@
	@ assignable a[*];
	@
	@ ensures TArrays.eq(a, 0, a.length, val); 
	@*/
	public static void fill0(int[] a, int val) {
		int[] old_a = new int[a.length];
		for (int i = 0; i < a.length; i++) {
			old_a[i] = a[i];
		}

		int ic = 0;
		int len = a.length;

		int loop_counter_1 = 0;
		fill0_loop_invariant_1(ic, a, val, old_a);

		for (; ic < len;) {
			fill0_loop_invariant_1(ic, a, val, old_a);
			if (loop_counter_1 < 10) {
				loop_counter_1++;
			}
			a[ic] = val;
			ic++;
			fill0_loop_invariant_1(ic, a, val, old_a);
		}
		fill0_loop_invariant_1(ic, a, val, old_a);
	}

	private static void fill0_loop_invariant_1(int ic, int[] a, int val,
			int[] old_a) {
		/*STUB*/
	}

	public static void sort(int[] a, int fromIndex, int toIndex) {
	}

	/*@
	@ assignable a[*];
	@ ensures (\forall int i; 0 <= i && i < a.length; a[i] != null);
	@ ensures (\forall int i; 0 <= i && i < a.length - 1; a[i].intValue() <= a[i + 1].intValue());
	@*/
	public static void sortInteger(java3.lang3.Integer[] a) {
		java3.lang3.Integer[] aux = (java3.lang3.Integer[]) a.clone();
		mergeSort(aux, a, 0, a.length, 0);
	}

	public static void sortInteger(java3.lang3.Integer[] a, int fromIndex,
			int toIndex) {
		java3.lang3.Integer[] aux = (java3.lang3.Integer[]) a.clone();
		mergeSort(aux, a, fromIndex, toIndex, -fromIndex);
	}

	public static void sort(int[] a) {
		sort1(a, 0, a.length);
	}

	/**
	 * Sorts the specified sub-array of integers into ascending order.
	 */
	private static void sort1(int x[], int off, int len) {
		// Insertion sort on smallest arrays
		if (len < 7) {
			insertionSort1(x, off, len);
			return;
		}

		// Choose a partition element, v
		int m = off + MathAux.signed_right_shift(len, 1); // Small arrays, middle element
		if (len > 7) {
			int l = off;
			int n = off + len - 1;
			if (len > 40) { // Big arrays, pseudomedian of 9
				int s = len / 8;
				l = med3(x, l, l + s, l + 2 * s);
				m = med3(x, m - s, m, m + s);
				n = med3(x, n - 2 * s, n - s, n);
			}
			m = med3(x, l, m, n); // Mid-size, med of 3
		}
		int v = x[m];

		// Establish Invariant: v* (<v)* (>v)* v*
		PartitionIndexes idxs = quicksortPartition(x, off, len, v);
		int a = idxs.a;
		int b = idxs.b;
		int c = idxs.c;
		int d = idxs.d;

		// Swap partition elements back to middle
		int s, n = off + len;
		s = Math.min(a - off, b - a);
		vecswap(x, off, b - s, s);
		s = Math.min(d - c, n - d - 1);
		vecswap(x, b, n - s, s);

		// Recursively sort non-partition-elements
		if ((s = b - a) > 1)
			sort1(x, off, s);
		if ((s = d - c) > 1)
			sort1(x, n - s, s);
	}

  /*@ 
	@ requires x != null;
	@ requires len >= 0;
	@ requires off >=0;
	@ requires off + len <= x.length;
	@
	@ assignable x[*]; 
	@
	@ ensures (\forall int i; 0 <= i && i < off ==> x[i] == \old(x[i]));
	@ ensures (\forall int i; off + len <= i && i < x.length ==> x[i] == \old(x[i]));
	@ ensures TArrays.eq(x, off, \result.a, v);
	@ ensures TArrays.less(x, \result.a, \result.b, v);
	@ ensures TArrays.grt(x, \result.c + 1, \result.d + 1, v);
	@ ensures TArrays.eq(x, \result.d + 1, off + len, v);
	@ ensures off <= \result.a;
	@ ensures \result.a <= \result.b;
	@ ensures \result.b == \result.c + 1;
	@ ensures \result.c <= \result.d;
	@ ensures \result.d < off + len;
	@*/
	private static PartitionIndexes quicksortPartition(int[] x, int off,
			int len, int v) {

		if (!(x != null))
			throw new AssertionError();

		if (!(len >= 0))
			throw new AssertionError();

		if (!(off >= 0))
			throw new AssertionError();

		if (!(off + len <= x.length))
			throw new AssertionError();

		int[] old_x = new int[x.length];
		for (int i = 0; i < x.length; i++) {
			old_x[i] = x[i];
		}

		int a = off, b = a, c = off + len - 1, d = c;
		int loop_counter_1 = 0;
		quicksortPartition_loop_invariant_1(x, off, len, v, a, b, c, d, old_x);
		while (true) {
			quicksortPartition_loop_invariant_1(x, off, len, v, a, b, c, d,
					old_x);
			if (loop_counter_1 < 10) {
				loop_counter_1++;
			}

			int loop_counter_2 = 0;
			quicksortPartition_loop_invariant_2(x, off, len, v, a, b, c, d,
					old_x);
			while (b <= c && x[b] <= v) {
				quicksortPartition_loop_invariant_2(x, off, len, v, a, b, c, d,
						old_x);
				if (loop_counter_2 < 10) {
					loop_counter_2++;
				}
				if (x[b] == v) {
					swap(x, a++, b);
				}
				b++;
				quicksortPartition_loop_invariant_2(x, off, len, v, a, b, c, d,
						old_x);
			}
			quicksortPartition_loop_invariant_2(x, off, len, v, a, b, c, d,
					old_x);

			int loop_counter_3 = 10;
			quicksortPartition_loop_invariant_3(x, off, len, v, a, b, c, d,
					old_x);
			while (c >= b && x[c] >= v) {
				quicksortPartition_loop_invariant_3(x, off, len, v, a, b, c, d,
						old_x);
				if (loop_counter_3 < 10) {
					loop_counter_3++;
				}

				if (x[c] == v) {
					swap(x, c, d--);
				}
				c--;
				quicksortPartition_loop_invariant_3(x, off, len, v, a, b, c, d,
						old_x);
			}
			quicksortPartition_loop_invariant_3(x, off, len, v, a, b, c, d,
					old_x);
			if (b > c) {
				break;
			}
			// //@ assert b <= c;
			// //@ assert x[b] > v && x[c] < v;
			swap(x, b++, c--);
			// //@ assert TArrays.eq(x, off, a, v);
			// //@ assert TArrays.less(x, a, b, v);
			// //@ assert TArrays.grt(x, c + 1, d + 1, v);
			// //@ assert TArrays.eq(x, d + 1, off + len, v);
			quicksortPartition_loop_invariant_1(x, off, len, v, a, b, c, d,
					old_x);
		}
		quicksortPartition_loop_invariant_1(x, off, len, v, a, b, c, d, old_x);
		PartitionIndexes idxs = new PartitionIndexes();
		idxs.a = a;
		idxs.b = b;
		idxs.c = c;
		idxs.d = d;
		return idxs;
	}

	private static void quicksortPartition_loop_invariant_3(int[] x, int off,
			int len, int v, int a, int b, int c, int d, int[] old_x) {
		/*STUB*/
	}

	private static void quicksortPartition_loop_invariant_2(int[] x, int off,
			int len, int v, int a, int b, int c, int d, int[] old_x) {
		/*STUB*/
	}

	private static void quicksortPartition_loop_invariant_1(int[] x, int off,
			int len, int v, int a, int b, int c, int d, int[] old_x) {
		/*STUB*/
	}

	/*@ 
	@ requires x != null;
	@ requires len >= 0;
	@ requires TArrays.within(x, off, off + len);
	@
	@ assignable x[*];
	@
	@ ensures TArrays.sorted(x, off, off + len);
	@*/
	private static void insertionSort1(int[] x, int off, int len) {

		if (!(x != null)) {
			return;
		}

		if (!(len >= 0)) {
			return;
		}

		if (!TArrays.within(x, off, off + len)) {
			return;
		}

		int[] old_x = new int[x.length];
		for (int i = 0; i < x.length; i++) {
			old_x[i] = x[i];
		}

		if (off < 0 || len > x.length || len < 0 || off > x.length
				|| off + len > x.length)
			throw new RuntimeException();

		if (len == 0)
			return;

		if (len == 1)
			return;

		int high = off + len;
		int loop_counter_1 = 0;

		int ic = off + 1;
		insertionSort1_loop_invariant_1(x, off, high, ic, old_x, len);
		while (ic < high) {
			insertionSort1_loop_invariant_1(x, off, high, ic, old_x, len);
			if (loop_counter_1 < 10) {
				loop_counter_1++;
			}
			int new_value = x[ic];

			x[ic] = x[ic - 1];

			int jc = ic;
			int loop_counter_2 = 0;
			insertionSort1_loop_invariant_2(x, off, high, ic, jc, new_value,
					old_x, len);
			while (jc > off && x[jc - 1] > new_value) {
				insertionSort1_loop_invariant_2(x, off, high, ic, jc,
						new_value, old_x, len);
				if (loop_counter_2 < 10) {
					loop_counter_2++;
				}

				x[jc] = x[jc - 1];
				jc--;
				insertionSort1_loop_invariant_2(x, off, high, ic, jc,
						new_value, old_x, len);
			}
			insertionSort1_loop_invariant_2(x, off, high, ic, jc, new_value,
					old_x, len);

			x[jc] = new_value;
			ic++;

			insertionSort1_loop_invariant_1(x, off, high, ic, old_x, len);
		}
		insertionSort1_loop_invariant_1(x, off, high, ic, old_x, len);

	}

	private static void insertionSort1_loop_invariant_2(int[] x, int off,
			int high, int ic, int jc, int new_value, int[] old_x, int len) {
		/*STUB*/
	}

	private static void insertionSort1_loop_invariant_1(int[] x, int off,
			int high, int ic, int[] old_x, int len) {
		/*STUB*/
	}

	/**
	 * Swaps x[a] with x[b].
	 */
	/*@ 
	@ requires x != null;
	@ requires TArrays.in_bound(x, a) && TArrays.in_bound(x, b); 
	@
	@ assignable x[a], x[b];
	@
	@ ensures x[a] == \old(x[b]);
	@ ensures x[b] == \old(x[a]);
	@ ensures (\forall int i; 0 <= i && i < x.length && i != a && i != b ==> x[i] == \old(x[i]));
	@*/
	private static void swap(int x[], int a, int b) {
		int t = x[a];
		x[a] = x[b];
		x[b] = t;
	}

	/**
	 * Returns the index of the median of the three indexed integers.
	 */
	private static int med3(int x[], int a, int b, int c) {
		return (x[a] < x[b] ? (x[b] < x[c] ? b : x[a] < x[c] ? c : a)
				: (x[b] > x[c] ? b : x[a] > x[c] ? c : a));
	}

	/**
	 * Swaps x[a .. (a+n-1)] with x[b .. (b+n-1)].
	 */
	/*@ 
	@ requires x != null;
	@ requires n >= 0;
	@ requires TArrays.within(x, a, a + n);
	@ requires TArrays.within(x, b, b + n);
	// intervals are disjoint
	@ requires a + n <= b || b + n <= a;
	@
	@ assignable x[*];
	@
	@ ensures (\forall int j; 0 <= j && j < (a < b ? a : b) ==> x[j] == \old(x[j]));
	@ ensures (\forall int j; (a > b ? a : b) + n <= j && j < x.length ==> x[j] == \old(x[j]));
	@ ensures (\forall int j; 0 <= j && j < n ==> x[a + j] == \old(x[b + j]));
	@ ensures (\forall int j; 0 <= j && j < n ==> x[b + j] == \old(x[a + j]));
	@*/
	private static void vecswap(int x[], int a, int b, int n) {
		int[] old_x = new int[x.length];
		for (int i = 0; i < x.length; i++) {
			old_x[i] = x[i];
		}

		int ic = 0;
		int loop_counter_1 = 0;

		vecswap_loop_invariant_1(ic, n, a, b, x, old_x);
		for (; ic < n;) {
			vecswap_loop_invariant_1(ic, n, a, b, x, old_x);
			if (loop_counter_1 < 10) {
				loop_counter_1++;
			}
			swap(x, a + ic, b + ic);
			ic++;
			vecswap_loop_invariant_1(ic, n, a, b, x, old_x);
		}
		vecswap_loop_invariant_1(ic, n, a, b, x, old_x);
	}

	private static void vecswap_loop_invariant_1(int ic, int n, int a, int b,
			int[] x, int[] old_x) {
		/*STUB*/
	}

	/**
	 * Check that fromIndex and toIndex are in range, and throw an appropriate
	 * exception if they aren't.
	 */
	/*@ normal_behavior
	  @ requires 0 <= fromIndex && fromIndex <= toIndex && toIndex <= arrayLen;
	  @
	  @ also exceptional_behavior
	  @ requires fromIndex > toIndex;
	  @ signals (java3.lang3.IllegalArgumentException ex) true;
	  @
	  @ also exceptional_behavior
	  @ requires fromIndex < 0 || toIndex > arrayLen;
	  @ signals (java3.lang3.ArrayIndexOutOfBoundsException ex) true;
	  @*/
	private static/*@ pure @*/void rangeCheck(int arrayLen, int fromIndex,
			int toIndex) throws java3.lang3.IllegalArgumentException,
			java3.lang3.ArrayIndexOutOfBoundsException {
		if (fromIndex > toIndex)
			throw new java3.lang3.IllegalArgumentException();
		if (fromIndex < 0)
			throw new java3.lang3.ArrayIndexOutOfBoundsException();
		if (toIndex > arrayLen)
			throw new java3.lang3.ArrayIndexOutOfBoundsException();
	}

	/**
	 * Tuning parameter: list size at or below which insertion sort will be
	 * used in preference to mergesort or quicksort.
	 */
	private static final int INSERTIONSORT_THRESHOLD = 7;

	/**
	 * Src is the source array that starts at index 0
	 * Dest is the (possibly larger) array destination with a possible offset
	 * low is the index in dest to start sorting
	 * high is the end index in dest to end sorting
	 * off is the offset to generate corresponding low, high in src
	 */

	private static void mergeSort(java3.lang3.Integer[] src,
			java3.lang3.Integer[] dest, int low, int high, int off) {
		int length = high - low;

		// Insertion sort on smallest arrays
		if (length < INSERTIONSORT_THRESHOLD) {
			insertionSort0(dest, low, high);
			return;
		}

		// Recursively sort halves of dest into src
		int destLow = low;
		int destHigh = high;
		low += off;
		high += off;
		int mid = MathAux.unsigned_right_shift(low + high, 1);
		mergeSort(dest, src, low, mid, -off);
		mergeSort(dest, src, mid, high, -off);

		// If list is already sorted, just copy from src to dest.  This is an
		// optimization that results in faster sorts for nearly ordered lists.
		if (src[mid - 1].intValue() <= src[mid].intValue()) {
			java3.lang3.System
					.arraycopyInteger(src, low, dest, destLow, length);
			return;
		}

		merge0(src, dest, low, mid, high, destLow, destHigh);
	}

	/*@ 
	@ requires src != null && dest != null;
	// Given the comment at the first instruction:
		@ requires TArrays.sorted(src, low, mid) && TArrays.sorted(src, mid, high);
	@ requires TArrays.within(src, low, high);
	@ requires low <= mid && mid <= high;
	@ requires TArrays.within(dest, destLow, destHigh);
	@ requires high - low == destHigh - destLow;
	@ requires TArrays.nonnull(src, low, high);
	// Source and destination are disjoint
	@ requires src == dest ==> high <= destLow || destHigh <= low;
	@
	@ assignable dest[*];
	@
	@ ensures TArrays.sorted(dest, destLow, destHigh);
	@*/
	private static void merge0(java3.lang3.Integer[] src,
			java3.lang3.Integer[] dest, final int low, final int mid,
			final int high, final int destLow, final int destHigh) {
		if (!(src != null && dest != null))
			return;
		if (!(TArrays.sorted(src, low, mid) && TArrays.sorted(src, mid, high)))
			return;
		if (!(TArrays.within(src, low, high)))
			return;
		if (!(low <= mid && mid <= high))
			return;
		if (!(TArrays.within(dest, destLow, destHigh)))
			return;
		if (!(high - low == destHigh - destLow))
			return;
		if (!(TArrays.nonnull(src, low, high)))
			return;
		if (!((src == dest ? (high <= destLow || destHigh <= low) : true)))
			return;

		java3.lang3.Integer[] old_dest = new java3.lang3.Integer[dest.length];
		for (int i = 0; i < dest.length; i++) {
			old_dest[i] = dest[i];
		}

		// Merge sorted halves (now in src) into dest
		int ic = destLow, p = low, q = mid;

		int loop_counter_1 = 0;

		merge0_loop_invariant_1(src, dest, low, mid, high, destLow, destHigh,
				p, q, ic, old_dest);
		//@ loop_invariant destLow <= ic;
		//@ loop_invariant ic <= destHigh;
		//@ loop_invariant destHigh - ic == (mid - p) + (high - q);
		//@ loop_invariant low <= p;
		//@ loop_invariant p <= mid;
		//@ loop_invariant mid <= q;
		//@ loop_invariant q <= high;
		//@ loop_invariant 0 <= low;
		//@ loop_invariant low <= high;
		//@ loop_invariant high <= src.length;
		//@ loop_invariant 0 <= destLow;
		//@ loop_invariant destLow <= destHigh;
		//@ loop_invariant destHigh <= dest.length;
		//@ loop_invariant TArrays.nonnull(src, p, mid);
		//@ loop_invariant TArrays.nonnull(src, q, high);
		//@ loop_invariant TArrays.nonnull(dest, destLow, ic);
		// Not using predicates for speed of static checking
		//@ loop_invariant (\forall int i, j; p <= i && i < j && j < mid ==> src[i].intValue() <= src[j].intValue());
		//@ loop_invariant (\forall int i, j; q <= i && i < j && j < high ==> src[i].intValue() <= src[j].intValue());
		//@ loop_invariant p < mid ==> (\forall int j; destLow <= j && j < ic ==> dest[j].intValue() <= src[p].intValue());
		//@ loop_invariant q < high ==> (\forall int j; destLow <= j && j < ic ==> dest[j].intValue() <= src[q].intValue());
		//@ loop_invariant (\forall int i, j; destLow <= i && i < j && j < ic ==> dest[i].intValue() <= dest[j].intValue());
		//merge_loop_invariant_1
		for (; ic < destHigh;) {
			merge0_loop_invariant_1(src, dest, low, mid, high, destLow,
					destHigh, p, q, ic, old_dest);
			if (loop_counter_1 < 10) {
				loop_counter_1++;
			}
			if (q >= high || p < mid
					&& (src[p].intValue() <= src[q].intValue()))
				dest[ic] = src[p++];
			else
				dest[ic] = src[q++];

			ic++;
			merge0_loop_invariant_1(src, dest, low, mid, high, destLow,
					destHigh, p, q, ic, old_dest);
		}
		merge0_loop_invariant_1(src, dest, low, mid, high, destLow, destHigh,
				p, q, ic, old_dest);
	}

	private static void merge0_loop_invariant_1(java3.lang3.Integer[] src,
			java3.lang3.Integer[] dest, int low, int mid, int high,
			int destLow, int destHigh, int p, int q, int ic,
			java3.lang3.Integer[] old_dest) {
		/*STUB*/
	}

	/*@ 
	@ requires dest != null;
		@ requires TArrays.within(dest, low, high);
		@ requires TArrays.nonnull(dest, low, high);
		@
		@ assignable dest[*];
		@
		@ ensures TArrays.sorted(dest, low, high);
		@*/
	private static void insertionSort0(java3.lang3.Integer[] dest, int low,
			int high) {

		if (!(dest != null)) {
			return;
		}

		if (!TArrays.within(dest, low, high)) {
			return;
		}

		if (!TArrays.nonnull(dest, low, high)) {
			return;
		}

		java3.lang3.Integer[] old_dest = new java3.lang3.Integer[dest.length];
		for (int i = 0; i < dest.length; i++) {
			old_dest[i] = dest[i];
		}

		if (dest.length == 0)
			return;
		if (dest.length == 1)
			return;

		if (low==high) {
			return;
		}

		int loop_counter_1 = 0;

		int ic = low + 1;

		insertionSort0_loop_invariant_1(ic, high, dest, low, old_dest);
		while (ic < high) {
			insertionSort0_loop_invariant_1(ic, high, dest, low, old_dest);

			if (loop_counter_1 < 10) {
				loop_counter_1++;
			}
			java3.lang3.Integer new_value = dest[ic];

			dest[ic] = dest[ic - 1];

			int jc = ic;
			int loop_counter_2 = 0;

			insertionSort0_loop_invariant_2(dest, ic, jc, low, high, new_value,
					old_dest);
			while (jc > low && dest[jc - 1].intValue() > new_value.intValue()) {
				insertionSort0_loop_invariant_2(dest, ic, jc, low, high,
						new_value, old_dest);
				if (loop_counter_2 < 10) {
					loop_counter_2++;
				}

				dest[jc] = dest[jc - 1];
				jc--;
				insertionSort0_loop_invariant_2(dest, ic, jc, low, high,
						new_value, old_dest);
			}
			insertionSort0_loop_invariant_2(dest, ic, jc, low, high, new_value,
					old_dest);

			dest[jc] = new_value;
			ic++;
			insertionSort0_loop_invariant_1(ic, high, dest, low, old_dest);
		}
		insertionSort0_loop_invariant_1(ic, high, dest, low, old_dest);

	}

	private static void insertionSort0_loop_invariant_2(
			java3.lang3.Integer[] dest, int ic, int jc, int low, int high,
			java3.lang3.Integer new_value, java3.lang3.Integer[] old_dest) {
		/*STUB*/
	}

	private static void insertionSort0_loop_invariant_1(int ic, int high,
			java3.lang3.Integer[] dest, int low, java3.lang3.Integer[] old_dest) {
		/*STUB*/
	}

	private static class PartitionIndexes {
		int a;
		int b;
		int c;
		int d;
	}

	/**
	 * Returns <tt>true</tt> if the two specified arrays of ints are
	 * <i>equal</i> to one another.  Two arrays are considered equal if both
	 * arrays contain the same number of elements, and all corresponding pairs
	 * of elements in the two arrays are equal.  In other words, two arrays
	 * are equal if they contain the same elements in the same order.  Also,
	 * two array references are considered equal if both are <tt>null</tt>.<p>
	 *
	 * @param a one array to be tested for equality
	 * @param a2 the other array to be tested for equality
	 * @return <tt>true</tt> if the two arrays are equal
	 */
	/*@ 
	  @ ensures \result <==> (a == a2) || 
	  @                      (a != null && a2 != null && TArrays.eq(a, 0, a.length, a2, 0, a2.length)); 
	  @*/
	public static/*@ pure @*/boolean equals0(int[] a, int[] a2) {
		if (a == a2)
			return true;
		if (a == null || a2 == null)
			return false;

		int length = a.length;
		if (a2.length != length)
			return false;

		int ic = 0;

		equals0_loop_invariant_1(ic, a, a2);
		for (; ic < length;) {
			equals0_loop_invariant_1(ic, a, a2);
			if (a[ic] != a2[ic]) {
				equals0_loop_invariant_1(ic, a, a2);
				return false;
			}
			ic++;
			equals0_loop_invariant_1(ic, a, a2);
		}
		equals0_loop_invariant_1(ic, a, a2);

		return true;
	}

	private static void equals0_loop_invariant_1(int ic, int[] a, int[] a2) {
		/*STUB*/
	}

	/**
	 * Returns a hash code based on the contents of the specified array.
	 * For any two non-null <tt>int</tt> arrays <tt>a</tt> and <tt>b</tt>
	 * such that <tt>Arrays.equals(a, b)</tt>, it is also the case that
	 * <tt>Arrays.hashCode(a) == Arrays.hashCode(b)</tt>.
	 *
	 * <p>The value returned by this method is the same value that would be
	 * obtained by invoking the {@link List#hashCode() <tt>hashCode</tt>}
	 * method on a {@link List} containing a sequence of {@link Integer}
	 * instances representing the elements of <tt>a</tt> in the same order.
	 * If <tt>a</tt> is <tt>null</tt>, this method returns 0.
	 *
	 * @param a the array whose hash value to compute
	 * @return a content-based hash code for <tt>a</tt>
	 * @since 1.5
	 */
	/*  */
	/*@ 
	  @ ensures a == null ==> \result == 0; 
	  @*/
	public static/*@ pure @*/int hashCode0(int a[]) {
		if (a == null)
			return 0;

		int result = 1;
		int ic = 0;
		int loop_counter_1 = 0;
		hashCode0_loop_invariant_1(a, ic);
		for (; ic < a.length;) {
			hashCode0_loop_invariant_1(a, ic);
			if (loop_counter_1 < 10) {
				loop_counter_1++;
			}
			int element = a[ic];
			result = 31 * result + element;
			ic++;
			hashCode0_loop_invariant_1(a, ic);
		}
		hashCode0_loop_invariant_1(a, ic);
		return result;
	}

	private static void hashCode0_loop_invariant_1(int[] a, int ic) {
		/*STUB*/
	}

	/*@ 
	  @ ensures a == null ==> \result == 0; 
	  @*/
	public static/*@pure@*/int hashCodeInteger(java3.lang3.Integer a[]) {
		if (a == null)
			return 0;

		int result = 1;

		int ic = 0;
		int loop_counter_1 = 0;
		hashCodeInteger_loop_invariant_1(a, ic);
		for (; ic < a.length;) {
			hashCodeInteger_loop_invariant_1(a, ic);
			if (loop_counter_1 < 10) {
				loop_counter_1++;
			}
			java3.lang3.Integer element = a[ic];
			result = 31 * result + (element == null ? 0 : element.hashCode());
			ic++;
			hashCodeInteger_loop_invariant_1(a, ic);
		}
		hashCodeInteger_loop_invariant_1(a, ic);
		return result;
	}

	private static void hashCodeInteger_loop_invariant_1(
			java3.lang3.Integer[] a, int ic) {
		/*STUB*/
	}

	/**
	 * Assigns the specified Object reference to each element of the specified
	 * array of Objects.
	 *
	 * @param a the array to be filled
	 * @param val the value to be stored in all elements of the array
	 * @throws ArrayStoreException if the specified value is not of a
	 *         runtime type that can be stored in the specified array
	 */
	/*@ 
	  @ requires a != null;
	  @
	  @ assignable a[*];
	  @
	  @ ensures val != null ==> TArrays.eq(a, 0, a.length, val); 
	  @ ensures val != null ==> TArrays.nonnull(a, 0, a.length); 
	  @ ensures val == null ==> TArrays.eqnull(a, 0, a.length); 
	  @*/
	public static void fillInteger0(java3.lang3.Integer[] a,
			java3.lang3.Integer val) {
		java3.lang3.Integer[] old_a = new java3.lang3.Integer[a.length];
		for (int i = 0; i < a.length; i++) {
			old_a[i] = a[i];
		}

		int ic = 0, len = a.length;
		int loop_counter_1 = 0;

		fillInteger0_loop_invariant_1(a, val, ic, old_a);
		for (; ic < len;) {
			fillInteger0_loop_invariant_1(a, val, ic, old_a);
			if (loop_counter_1 < 10) {
				loop_counter_1++;
			}
			a[ic] = val;
			ic++;
			fillInteger0_loop_invariant_1(a, val, ic, old_a);
		}
		fillInteger0_loop_invariant_1(a, val, ic, old_a);
	}

	private static void fillInteger0_loop_invariant_1(java3.lang3.Integer[] a,
			java3.lang3.Integer val, int ic, java3.lang3.Integer[] old_a) {
		/*STUB*/
	}

	/*@ normal_behavior
	  @ requires a != null;
	  @ requires TArrays.within(a, fromIndex, toIndex);
	  @ assignable a[*];
	  @ ensures (\forall int i; 0 <= i && i < fromIndex ==> a[i] == \old(a[i]));
	  @ ensures (\forall int i; toIndex <= i && i < a.length ==> a[i] == \old(a[i]));
	  @ ensures val != null ==> TArrays.nonnull(a, fromIndex, toIndex);
	  @ ensures val != null ==> TArrays.eq(a, fromIndex, toIndex, val);
	  @ ensures val == null ==> TArrays.eqnull(a, fromIndex, toIndex);
	  @
	  @ also exceptional_behavior
	  @ requires fromIndex > toIndex;
	  @ signals (java3.lang3.IllegalArgumentException ex) true;
	  @
	  @ also exceptional_behavior
	  @ requires fromIndex < 0 || toIndex > a.length;
	  @ signals (java3.lang3.ArrayIndexOutOfBoundsException  ex) true;
	  @
	  @ also exceptional_behavior
	  @ requires a == null;
	  @ signals (NullPointerException ex) true;
	  @*/
	public static void fillInteger1(java3.lang3.Integer[] a, int fromIndex,
			int toIndex, java3.lang3.Integer val)
			throws java3.lang3.ArrayIndexOutOfBoundsException,
			NullPointerException, java3.lang3.IllegalArgumentException {
		if (a == null)
			throw new NullPointerException();

		java3.lang3.Integer[] old_a = new java3.lang3.Integer[a.length];
		for (int i = 0; i < a.length; i++) {
			old_a[i] = a[i];
		}

		int ic = fromIndex;
		rangeCheck(a.length, fromIndex, toIndex);

		int loop_counter_1 = 0;

		fillInteger1_loop_invariant_1(a, fromIndex, toIndex, ic, val, old_a);
		for (; ic < toIndex;) {
			fillInteger1_loop_invariant_1(a, fromIndex, toIndex, ic, val, old_a);
			if (loop_counter_1 < 10) {
				loop_counter_1++;
			}
			a[ic] = val;
			ic++;
			fillInteger1_loop_invariant_1(a, fromIndex, toIndex, ic, val, old_a);
		}
		fillInteger1_loop_invariant_1(a, fromIndex, toIndex, ic, val, old_a);
	}

	private static void fillInteger1_loop_invariant_1(java3.lang3.Integer[] a,
			int fromIndex, int toIndex, int ic, java3.lang3.Integer val,
			java3.lang3.Integer[] old_a) {
		/*STUB*/
	}

	/**
	 * The only purpose of this method is allowing 
	 * Chicory to access the private fields of this class.
	 * 
	 */
	private void unreachable_method() {
	}

}